Package: fasterRaster
Type: Package
Title: Faster Raster and Spatial Vector Processing Using 'GRASS'
Version: 8.4.1.2
Date: 2026-04-17
Authors@R: 
	c(
		person(
			given = c('Adam', 'B.'),
			family = 'Smith',
			role = c('cre', 'aut'),
			email = 'adam.smith@mobot.org',
			comment = c(ORCID = '0000-0002-6420-1659')
		)
	)
Maintainer: Adam B. Smith <adam.smith@mobot.org>
Description: Processing of large-in-memory/large-on disk rasters and spatial
	vectors using 'GRASS' <https://grass.osgeo.org/>. Most functions in the
	'terra' package are	recreated. Processing of medium-sized and smaller
	spatial objects will nearly always be faster using 'terra' or 'sf', but
	for large-in-memory/large-on-disk objects, 'fasterRaster' may be faster.
	To use most of the functions, you must have the stand-alone version (not
	the 'OSGeoW4' installer version) of 'GRASS' 8.0 or higher.
Depends: R (>= 4.0.0)
Imports: data.table (>= 1.14.8), DT, graphics, grDevices, methods,
        omnibus (>= 1.2.15), rgrass (>= 0.3-9), sf, shiny, terra (>=
        1.7), utils
Suggests: knitr, rmarkdown
License: GPL (>= 3)
SystemRequirements: GRASS (>= 8)
URL: https://github.com/adamlilith/fasterRaster,
        https://adamlilith.github.io/fasterRaster/
BugReports: https://github.com/adamlilith/fasterRaster/issues
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
LazyLoad: yes
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-17 21:08:09 UTC; adame
Author: Adam B. Smith [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-6420-1659>)
Repository: CRAN
Date/Publication: 2026-04-17 21:50:02 UTC
Built: R 4.6.0; ; 2026-04-29 17:20:19 UTC; unix
