# Generated by roxygen2: do not edit by hand

export(.g.proj)
export(.g.region)
export(addons)
export(appFuns)
export(fastData)
export(faster)
export(grassHelp)
export(grassInfo)
export(grassStarted)
export(installAddon)
export(is.polygons)
export(makeGRaster)
export(makeGVector)
export(mow)
export(pcs)
export(print)
export(removeAddon)
export(seqToSQL)
export(st_coordinates)
export(st_crs)
export(sun)
exportClasses(GLocation)
exportClasses(GRaster)
exportClasses(GRegion)
exportClasses(GSpatial)
exportClasses(GVector)
exportMethods("$")
exportMethods("$<-")
exportMethods("%in%")
exportMethods("%notin%")
exportMethods("[")
exportMethods("[<-")
exportMethods("[[")
exportMethods("[[<-")
exportMethods("activeCat<-")
exportMethods("add<-")
exportMethods("addCats<-")
exportMethods("addTable<-")
exportMethods("levels<-")
exportMethods("names<-")
exportMethods(.workDir)
exportMethods(Arith)
exportMethods(Compare)
exportMethods(E)
exportMethods(Logic)
exportMethods(N)
exportMethods(S)
exportMethods(W)
exportMethods(abs)
exportMethods(acos)
exportMethods(activeCat)
exportMethods(activeCats)
exportMethods(addCats)
exportMethods(aggregate)
exportMethods(allNA)
exportMethods(anyNA)
exportMethods(app)
exportMethods(appCheck)
exportMethods(as.contour)
exportMethods(as.data.frame)
exportMethods(as.data.table)
exportMethods(as.doub)
exportMethods(as.float)
exportMethods(as.int)
exportMethods(as.lines)
exportMethods(as.points)
exportMethods(as.polygons)
exportMethods(asin)
exportMethods(atan)
exportMethods(atan2)
exportMethods(bioclims)
exportMethods(bottom)
exportMethods(breakPolys)
exportMethods(buffer)
exportMethods(c)
exportMethods(catNames)
exportMethods(categories)
exportMethods(cats)
exportMethods(ceiling)
exportMethods(cellSize)
exportMethods(centroids)
exportMethods(classify)
exportMethods(clump)
exportMethods(clusterPoints)
exportMethods(colbind)
exportMethods(combineLevels)
exportMethods(compareGeom)
exportMethods(complete.cases)
exportMethods(compositeRGB)
exportMethods(concats)
exportMethods(connectors)
exportMethods(convHull)
exportMethods(coordRef)
exportMethods(cos)
exportMethods(count)
exportMethods(crds)
exportMethods(crop)
exportMethods(crs)
exportMethods(datatype)
exportMethods(delaunay)
exportMethods(denoise)
exportMethods(dim)
exportMethods(dim3d)
exportMethods(disagg)
exportMethods(distance)
exportMethods(dropRows)
exportMethods(dropTable)
exportMethods(droplevels)
exportMethods(erase)
exportMethods(exp)
exportMethods(expanse)
exportMethods(ext)
exportMethods(extend)
exportMethods(extract)
exportMethods(fast)
exportMethods(fillHoles)
exportMethods(fillNAs)
exportMethods(fixBridges)
exportMethods(fixDangles)
exportMethods(fixLines)
exportMethods(floor)
exportMethods(flow)
exportMethods(flowPath)
exportMethods(focal)
exportMethods(fractalRast)
exportMethods(fragmentation)
exportMethods(freq)
exportMethods(geomorphons)
exportMethods(geomtype)
exportMethods(global)
exportMethods(grassGUI)
exportMethods(grid)
exportMethods(head)
exportMethods(hexagons)
exportMethods(hillshade)
exportMethods(hist)
exportMethods(horizonHeight)
exportMethods(init)
exportMethods(interpIDW)
exportMethods(interpSplines)
exportMethods(intersect)
exportMethods(is.2d)
exportMethods(is.3d)
exportMethods(is.cell)
exportMethods(is.doub)
exportMethods(is.factor)
exportMethods(is.float)
exportMethods(is.int)
exportMethods(is.lines)
exportMethods(is.lonlat)
exportMethods(is.na)
exportMethods(is.points)
exportMethods(kernel)
exportMethods(kurtosis)
exportMethods(layerCor)
exportMethods(levels)
exportMethods(ln)
exportMethods(log)
exportMethods(log10)
exportMethods(log10p)
exportMethods(log1p)
exportMethods(log2)
exportMethods(logic)
exportMethods(longlat)
exportMethods(mask)
exportMethods(maskNA)
exportMethods(match)
exportMethods(max)
exportMethods(mean)
exportMethods(median)
exportMethods(merge)
exportMethods(min)
exportMethods(minmax)
exportMethods(missing.cases)
exportMethods(missingCats)
exportMethods(mmode)
exportMethods(multivarEnvSim)
exportMethods(nacell)
exportMethods(names)
exportMethods(ncell)
exportMethods(ncell3d)
exportMethods(ncol)
exportMethods(ndepth)
exportMethods(neighborhoodMatrix)
exportMethods(neighbourhoodMatrix)
exportMethods(ngeom)
exportMethods(nlevels)
exportMethods(nlyr)
exportMethods(noise)
exportMethods(nonnacell)
exportMethods(not.na)
exportMethods(nrow)
exportMethods(nsubgeom)
exportMethods(nunique)
exportMethods(pairs)
exportMethods(plot)
exportMethods(plotRGB)
exportMethods(predict)
exportMethods(princomp)
exportMethods(project)
exportMethods(quantile)
exportMethods(rNormRast)
exportMethods(rSpatialDepRast)
exportMethods(rUnifRast)
exportMethods(rWalkRast)
exportMethods(range)
exportMethods(rast)
exportMethods(rasterize)
exportMethods(rbind)
exportMethods(regress)
exportMethods(remove0)
exportMethods(removeAngles)
exportMethods(removeBridges)
exportMethods(removeDangles)
exportMethods(removeDupCentroids)
exportMethods(removeDups)
exportMethods(removeSmallPolys)
exportMethods(reorient)
exportMethods(replaceNAs)
exportMethods(res)
exportMethods(res3d)
exportMethods(resample)
exportMethods(round)
exportMethods(ruggedness)
exportMethods(rvoronoi)
exportMethods(sampleRast)
exportMethods(scale)
exportMethods(scalepop)
exportMethods(sdpop)
exportMethods(segregate)
exportMethods(selectRange)
exportMethods(show)
exportMethods(simplifyGeom)
exportMethods(sin)
exportMethods(sineRast)
exportMethods(skewness)
exportMethods(smoothGeom)
exportMethods(snap)
exportMethods(sources)
exportMethods(spatSample)
exportMethods(sqrt)
exportMethods(st_as_sf)
exportMethods(st_buffer)
exportMethods(st_crs)
exportMethods(stdev)
exportMethods(streams)
exportMethods(stretch)
exportMethods(subset)
exportMethods(subst)
exportMethods(sum)
exportMethods(summary)
exportMethods(tail)
exportMethods(tan)
exportMethods(terrain)
exportMethods(thinLines)
exportMethods(thinPoints)
exportMethods(tiles)
exportMethods(top)
exportMethods(topology)
exportMethods(trim)
exportMethods(trunc)
exportMethods(union)
exportMethods(unscale)
exportMethods(update)
exportMethods(var)
exportMethods(varpop)
exportMethods(vect)
exportMethods(vegIndex)
exportMethods(voronoi)
exportMethods(wetness)
exportMethods(which.max)
exportMethods(which.min)
exportMethods(writeRaster)
exportMethods(writeVector)
exportMethods(xor)
exportMethods(xres)
exportMethods(yres)
exportMethods(zext)
exportMethods(zonal)
exportMethods(zonalGeog)
exportMethods(zres)
import(data.table, except = shift)
import(sf)
import(terra, except = shift)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(graphics,par)
importFrom(terra,median)
