#' @name madRivers
#'
#' @title Major rivers in a selected portion of Madagascar
#'
#' @description Spatial lines object of major rivers in a portion of Madagascar.
#'
#' @docType data
#'
#' @usage data(madRivers)
#'
#' @format An object of class `sf`.
#'
#' @keywords Madagascar
#'
#' @source [DIVA-GIS](https://diva-gis.org/)
#'
#' @example man/examples/ex_fastData.r
#'
NULL
