#' @name madCoast
#'
#' @title Shapefile of a portion of the coastline of Madagascar
#'
#' @description Borders of a selected portion of Madagascar
#'
#' @docType data
#'
#' @format ESRI Shapefile.
#'
#' @keywords Madagascar
#'
#' @source [Database of Global Administrative Areas Version 2.8 (GADM)](https://gadm.org)
#'
#' @example man/examples/ex_fastData.r
#'
NULL
