#' @name madDypsis
#'
#' @title Spatial points vector of records of Dypsis in eastern Madagascar
#'
#' @description Spatial points vector of herbarium specimens and observations of plants in the genus *Dypsis* (slender, evergreen palms) from a portion of eastern Madagascar.
#'
#' @docType data
#'
#' @usage data(madDypsis)
#'
#' @format An object of class `sf`.
#'
#' @keywords Dypsis Madagascar
#'
#' @source [Global Biodiversity Information Facility (GBIF)](https://gbif.org)
#'
#' @example man/examples/ex_fastData.r
#'
NULL
