#' @name madElev
#'
#' @title Elevation raster for an eastern portion of Madagascar
#'
#' @description Elevation raster for an eastern portion of Madagascar.
#'
#' @docType data
#'
#' @format An object of class `SpatRaster`. Values are mean meters above sea level.
#'
#' @keywords elevation Madagascar
#'
#' @source [WorldClim Version 2.1](https://worldclim.org)
#'
#' @example man/examples/ex_fastData.r
#'
NULL
