#' @name madCoverCats
#'
#' @title Table of land cover classes for an eastern portion of Madagascar
#'
#' @description This data frame corresponds to the [madCover][madCover] raster, which represents land cover for an eastern portion of Madagascar. Note that the land cover classes have been simplified, so this table and raster should *not* be used for "real" analyses.
#'
#' @docType data
#'
#' @format An object of class `data.frame`.
#'
#' @keywords Madagascar
#'
#' @seealso [madCover], `vignette("GRasters", package = "fasterRaster")`
#'
#' @references  Arino O., P. Bicheron, F. Achard, J. Latham, R. Witt and J.-L. Weber. 2008. GlobCover: The most detailed portrait of Earth. European Space Agency Bulletin 136:25-31. [http://due.esrin.esa.int](http://due.esrin.esa.int).
#'
#' @source [http://due.esrin.esa.int](http://due.esrin.esa.int)
#'
#' @example man/examples/ex_fastData.r
#'
NULL
