#' @name appFunsTable
#'
#' @title Functions that can be used in app()
#'
#' @description This is a table of functions that can be used in the [app()] function, their **R** equivalents, and the [datatype()] they return. You can view this table using `?appFunsTable` or as a searchable, sortable **Shiny** table using [appFuns()].
#'
#' @docType data
#'
#' @format A `data.frame`.
#'
#' @keywords app
#'
#' @source [OSGeo](https://grass.osgeo.org)
#'
#' @example man/examples/ex_app.r
#'
NULL
