#' @name madTmin
#'
#' @title Rasters of average monthly minimum temperature for an eastern portion of Madagascar
#'
#' @description Rasters of minimum temperature for an eastern portion of Madagascar from WorldClim 2.1 at ~3.33 arcminute resolution projected to the Tananarive (Paris)/Laborde Grid coordinate reference system. Values represent monthly averages across 1970-2000. Units are in degrees C. These should not be used for formal analysis.
#'
#' @docType data
#'
#' @format An object of class `SpatRaster`.
#'
#' @keywords climate Madagascar
#'
#' @references Fick, S.E. and Hijmans, R.J.  2017.  WorldClim 2: New 1-km spatial resolution climate surfaces for global land areas.  International Journal of Climatology 37:4302-4315. \doi{10.1002/joc.5086}
#'
#' @source \doi{https://dx.doi.org/10.1002/joc.5086}
#'
#' @example man/examples/ex_fastData.r
#'
NULL
